@@ -56,7 +56,6 @@ AppletInterface::AppletInterface(DeclarativeAppletScript *script, const QVariant
       m_toolTipItem(0),
       m_args(args),
       m_backgroundHints(Plasma::Types::StandardBackground),
-      m_busy(false),
       m_hideOnDeactivate(true),
       m_oldKeyboardShortcut(0),
       m_dummyNativeInterface(0),
@@ -67,7 +66,7 @@ AppletInterface::AppletInterface(DeclarativeAppletScript *script, const QVariant
     connect(this, &AppletInterface::configNeedsSaving,
             applet(), &Plasma::Applet::configNeedsSaving);
     connect(applet(), &Plasma::Applet::immutabilityChanged,
-            this, &AppletInterface::immutableChanged);
+            this, &AppletInterface::immutabilityChanged);
     connect(applet(), &Plasma::Applet::userConfiguringChanged,
             this, &AppletInterface::userConfiguringChanged);
 
@@ -80,6 +79,15 @@ AppletInterface::AppletInterface(DeclarativeAppletScript *script, const QVariant
     connect(applet(), &Plasma::Applet::destroyedChanged,
             this, &AppletInterface::destroyedChanged);
 
+    connect(applet(), &Plasma::Applet::titleChanged,
+            this, &AppletInterface::titleChanged);
+
+    connect(applet(), &Plasma::Applet::iconChanged,
+            this, &AppletInterface::iconChanged);
+
+    connect(applet(), &Plasma::Applet::busyChanged,
+            this, &AppletInterface::busyChanged);
+
     connect(applet(), &Plasma::Applet::activated,
             this, &AppletInterface::activated);
 
@@ -111,53 +119,6 @@ AppletInterface::AppletInterface(DeclarativeAppletScript *script, const QVariant
     });
 }
 
-AppletInterface::AppletInterface(Plasma::Applet *a, const QVariantList &args, QQuickItem *parent)
-    : AppletQuickItem(a, parent),
-      m_actionSignals(0),
-      m_configuration(0),
-      m_appletScriptEngine(0),
-      m_toolTipTextFormat(0),
-      m_args(args),
-      m_backgroundHints(Plasma::Types::StandardBackground),
-      m_busy(false),
-      m_hideOnDeactivate(true),
-      m_oldKeyboardShortcut(0),
-      m_dummyNativeInterface(0),
-      m_positionBeforeRemoval(QPointF(-1, -1))
-{
-    qmlRegisterType<QAction>();
-
-    connect(this, &AppletInterface::configNeedsSaving,
-            applet(), &Plasma::Applet::configNeedsSaving);
-    connect(applet(), &Plasma::Applet::immutabilityChanged,
-            this, &AppletInterface::immutableChanged);
-    connect(applet(), &Plasma::Applet::userConfiguringChanged,
-            this, &AppletInterface::userConfiguringChanged);
-    connect(applet(), &Plasma::Applet::configurationRequiredChanged,
-            this, [this](bool needsConfig, const QString &reason) {
-                emit configurationRequiredChanged();
-                emit configurationRequiredReasonChanged();
-            });
-
-    connect(applet(), &Plasma::Applet::statusChanged,
-            this, &AppletInterface::statusChanged);
-
-    connect(applet(), &Plasma::Applet::destroyedChanged,
-            this, &AppletInterface::destroyedChanged);
-
-    connect(appletScript(), &DeclarativeAppletScript::formFactorChanged,
-            this, &AppletInterface::formFactorChanged);
-    connect(appletScript(), &DeclarativeAppletScript::locationChanged,
-            this, &AppletInterface::locationChanged);
-    connect(appletScript(), &DeclarativeAppletScript::contextChanged,
-            this, &AppletInterface::contextChanged);
-
-    if (applet()->containment()) {
-        connect(applet()->containment(), &Plasma::Containment::screenChanged,
-                this, &ContainmentInterface::screenChanged);
-    }
-}
-
 AppletInterface::~AppletInterface()
 {
 }
@@ -181,11 +142,19 @@ void AppletInterface::init()
     emit busyChanged();
 
     applet()->updateConstraints(Plasma::Types::UiReadyConstraint);
+
     connect(applet(), &Plasma::Applet::activated,
     [ = ]() {
-        setExpanded(true);
-        if (QQuickItem *i = qobject_cast<QQuickItem *>(fullRepresentationItem())) {
-            i->setFocus(true, Qt::ShortcutFocusReason);
+        // in case the applet doesn't want to get shrinked on reactivation,
+        // we always expand it again (only in order to conform with legacy behaviour)
+        bool activate = !( isExpanded() && isActivationTogglesExpanded() );
+
+        setExpanded(activate);
+        if (activate) {
+            if (QQuickItem *i = qobject_cast<QQuickItem *>(fullRepresentationItem())) {
+                // Bug 372476: never pull focus away from it, only setFocus(true)
+                i->setFocus(true, Qt::ShortcutFocusReason);
+            }
         }
     });
 
@@ -259,7 +228,7 @@ uint AppletInterface::id() const
 
 QString AppletInterface::pluginName() const
 {
-    return applet()->pluginInfo().isValid() ? applet()->pluginInfo().pluginName() : QString();
+    return applet()->pluginMetaData().isValid() ? applet()->pluginMetaData().pluginId() : QString();
 }
 
 QString AppletInterface::icon() const
@@ -321,8 +290,8 @@ void AppletInterface::setToolTipMainText(const QString &text)
 
 QString AppletInterface::toolTipSubText() const
 {
-    if (m_toolTipSubText.isNull() && applet()->pluginInfo().isValid()) {
-        return applet()->pluginInfo().comment();
+    if (m_toolTipSubText.isNull() && applet()->pluginMetaData().isValid()) {
+        return applet()->pluginMetaData().description();
     } else {
         return m_toolTipSubText;
     }
@@ -370,26 +339,21 @@ void AppletInterface::setToolTipItem(QQuickItem *toolTipItem)
         return;
     }
 
+    m_toolTipItem = toolTipItem;
     connect(m_toolTipItem.data(), &QObject::destroyed,
             this, &AppletInterface::toolTipItemChanged);
 
-    m_toolTipItem = toolTipItem;
     emit toolTipItemChanged();
 }
 
 bool AppletInterface::isBusy() const
 {
-    return m_busy;
+    return applet()->isBusy();
 }
 
 void AppletInterface::setBusy(bool busy)
 {
-    if (m_busy == busy) {
-        return;
-    }
-
-    m_busy = busy;
-    emit busyChanged();
+    applet()->setBusy(busy);
 }
 
 Plasma::Types::BackgroundHints AppletInterface::backgroundHints() const
@@ -524,6 +488,11 @@ bool AppletInterface::immutable() const
     return applet()->immutability() != Plasma::Types::Mutable;
 }
 
+Plasma::Types::ImmutabilityType AppletInterface::immutability() const
+{
+    return applet()->immutability();
+}
+
 bool AppletInterface::userConfiguring() const
 {
     return applet()->isUserConfiguring();
@@ -541,7 +510,7 @@ int AppletInterface::apiVersion() const
     if (plugins.isEmpty()) {
         return -1;
     }
-    
+
     return plugins.first().value(QStringLiteral("X-KDE-PluginInfo-Version")).toInt();
 }
 
@@ -659,7 +628,13 @@ void AppletInterface::setConfigurationRequiredReason(const QString &reason)
 
 QString AppletInterface::downloadPath(const QString &file)
 {
-    const QString downloadDir = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) + "/Plasma/" + applet()->pluginInfo().pluginName() + '/';
+    Q_UNUSED(file);
+    return downloadPath();
+}
+
+QString AppletInterface::downloadPath() const
+{
+    const QString downloadDir = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) + "/Plasma/" + applet()->pluginMetaData().pluginId() + '/';
 
     if (!QFile::exists(downloadDir)) {
         QDir dir(QChar('/'));
@@ -671,7 +646,7 @@ QString AppletInterface::downloadPath(const QString &file)
 
 QStringList AppletInterface::downloadedFiles() const
 {
-    const QString downloadDir = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) + "/Plasma/" + applet()->pluginInfo().pluginName() + '/';
+    const QString downloadDir = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) + "/Plasma/" + applet()->pluginMetaData().pluginId() + '/';
     QDir dir(downloadDir);
     return dir.entryList(QDir::Files | QDir::NoSymLinks | QDir::Readable);
 }
@@ -689,6 +664,49 @@ void AppletInterface::executeAction(const QString &name)
     }
 }
 
+QVariantList AppletInterface::availableScreenRegion() const
+{
+    QVariantList regVal;
+
+    if (!applet()->containment() || !applet()->containment()->corona()) {
+        return regVal;
+    }
+
+    QRegion reg = QRect(0, 0, width(), height());
+    int screenId = screen();
+    if (screenId > -1) {
+        reg = applet()->containment()->corona()->availableScreenRegion(screenId);
+    }
+
+    foreach (QRect rect, reg.rects()) {
+        //make it relative
+        QRect geometry = applet()->containment()->corona()->screenGeometry(screenId);
+        rect.moveTo(rect.topLeft() - geometry.topLeft());
+        regVal << QVariant::fromValue(QRectF(rect));
+    }
+    return regVal;
+}
+
+QRect AppletInterface::availableScreenRect() const
+{
+    if (!applet()->containment() || !applet()->containment()->corona()) {
+        return QRect();
+    }
+
+    QRect rect(0, 0, width(), height());
+
+    int screenId = screen();
+
+    if (screenId > -1) {
+        rect = applet()->containment()->corona()->availableScreenRect(screenId);
+        //make it relative
+        QRect geometry = applet()->containment()->corona()->screenGeometry(screenId);
+        rect.moveTo(rect.topLeft() - geometry.topLeft());
+    }
+
+    return rect;
+}
+
 bool AppletInterface::event(QEvent *event)
 {
     // QAction keyboard shortcuts cannot work with QML2 (and probably newver will
@@ -790,14 +808,17 @@ bool AppletInterface::eventFilter(QObject *watched, QEvent *event)
                 return true;
             }
 
-            QMenu desktopMenu;
+            QMenu *desktopMenu = new QMenu;
+            emit applet()->contextualActionsAboutToShow();
             ci->addAppletActions(desktopMenu, applet(), event);
 
-            if (!desktopMenu.isEmpty()) {
-                desktopMenu.exec(e->globalPos());
+            if (!desktopMenu->isEmpty()) {
+                desktopMenu->setAttribute(Qt::WA_DeleteOnClose);
+                desktopMenu->popup(e->globalPos());
                 return true;
             }
 
+            delete desktopMenu;
             return false;
         }
     }
