@@ -148,8 +148,8 @@ void WallpaperInterface::syncWallpaperPackage()
         m_configuration = new KDeclarative::ConfigPropertyMap(configScheme(), this);
     }
 
-    m_qmlObject->setSource(QUrl::fromLocalFile(m_pkg.filePath("mainscript")));
     m_qmlObject->rootContext()->setContextProperty(QStringLiteral("wallpaper"), this);
+    m_qmlObject->setSource(QUrl::fromLocalFile(m_pkg.filePath("mainscript")));
 
     const QString rootPath = m_pkg.metadata().property(QStringLiteral("X-Plasma-RootPath")).toString();
     if (!rootPath.isEmpty()) {
@@ -199,9 +199,7 @@ QList<QAction *> WallpaperInterface::contextualActions() const
 
 bool WallpaperInterface::supportsMimetype(const QString &mimetype) const
 {
-    //FIXME: a less brutal way? packages should have valid KService :/
-    KDesktopFile desktop(m_pkg.path() + "/" + "metadata.desktop");
-    return desktop.desktopGroup().readEntry<QStringList>("X-Plasma-DropMimeTypes", QStringList()).contains(mimetype);
+    return KPluginMetaData::readStringList(m_pkg.kPackage().metadata().rawData(), "X-Plasma-DropMimeTypes").contains(mimetype);
 }
 
 void WallpaperInterface::setUrl(const QUrl &url)
